@@ -36,12 +36,10 @@ else
 fi
 
 if [ "$AM_ERROR" = "1" ]; then
-	echo -e  '\E[31;m'
 	echo -n "Your automake version `automake --version | sed -n -e 's#[^0-9]* \([0-9]*\.[0-9]*\.[0-9]*\).*#\1#p'`"
 	echo " is older than the suggested one, $AM_1.$AM_2.$AM_3"
 	echo "Go on at your own risk. :-)"
 	echo
-	tput sgr0
 fi
 
 # Check autoconf version
@@ -60,14 +58,49 @@ else
 fi
 
 if [ "$AC_ERROR" = "1" ]; then
-	echo -e  '\E[31;m'
 	echo -n "Your autoconf version `autoconf --version | sed -n -e 's#[^0-9]* \([0-9]*\.[0-9]*\).*#\1#p'`"
 	echo " is older than the suggested one, $AC_1.$AC_2"
 	echo "Go on at your own risk. :-)"
 	echo
-	tput sgr0
 fi
 
+#Check for pkg-config
+if which pkg-config 1>/dev/null 2>&1; then
+	#pkg-config seems to be installed. Check for m4 macros:
+	tmpdir=`mktemp -d "./autogenXXXXXX"`
+	if [ -z ${tmpdir} ]; then
+		echo -n "Creating a temporary directory failed. "
+		echo 'Is mktemp in $PATH?'
+		echo
+		exit 1
+	fi
+
+	oldpwd=`pwd`
+	cd "${tmpdir}"
+	cat > configure.ac << _EOF_
+AC_INIT
+PKG_CHECK_MODULES(QtNetwork, [QtNetwork >= 4.4],  ac_qt="yes", ac_qt="no")
+_EOF_
+	aclocal
+	autoconf --force
+	grep -q PKG_CHECK_MODULES configure
+	PKG_MACRO_AVAILABLE=$?
+	cd "${oldpwd}"
+	rm -rf "${tmpdir}"
+	if [ ${PKG_MACRO_AVAILABLE} -eq 0 ]; then
+		echo -n "pkg-config installed, but autoconf is not able to find pkg.m4. "
+		echo "Unfortunately the generated configure would not work, so we stop here."
+		echo
+		exit 1
+	fi
+else
+	echo -n "pkg-config not found. "
+	echo "pkg-config is required to create a working configure, so we stop here."
+	echo
+	exit 1
+fi
+
+
 # Check libtool version
 if [ -z "$LIBTOOL" ] ; then
 	LIBTOOL="libtool"
@@ -87,12 +120,10 @@ else
 fi
 
 if [ "$LT_ERROR" = "1" ]; then
-	echo -e  '\E[31;m'
 	echo -n "Your libtool version `libtool --version | sed -n -e 's#[^0-9]* \([0-9]*\.[0-9]*\).*#\1#p'`"
 	echo " is older than the suggested one, $LT_1.$LT_2"
 	echo "Go on at your own risk. :-)"
 	echo
-	tput sgr0
 fi
 
 if [ -z "$LIBTOOLIZE" ] ; then
