@@ -1,4 +1,7 @@
-/* $Id: gpsd_report.c 6757 2009-12-07 20:29:57Z mledford $ */
+/*
+ * This file is Copyright (c) 2010 by the GPSD project
+ * BSD terms apply: see the file COPYING in the distribution root for details.
+ */
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -6,10 +9,11 @@
 
 
 # if __GNUC__ >= 3 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
-void gpsd_report(int errlevel UNUSED, const char *fmt, ... ) __attribute__ ((weak));
+void gpsd_report(int errlevel UNUSED, const char *fmt, ...)
+    __attribute__ ((weak));
 #endif
 
-void gpsd_report(int errlevel UNUSED, const char *fmt, ... )
+void gpsd_report(int errlevel UNUSED, const char *fmt, ...)
 /* stub logger for clients that don't supply one */
 {
     va_list ap;
