@@ -250,9 +250,12 @@ static int json_internal_read_object(const char *cp,
 		*pattr++ = '\0';
 		json_debug_trace((1, "Collected attribute name %s\n",
 				  attrbuf));
-		for (cursor = attrs; cursor->attribute != NULL; cursor++)
+		for (cursor = attrs; cursor->attribute != NULL; cursor++) {
+		    json_debug_trace((2, "Checking against %s\n",
+				      cursor->attribute));
 		    if (strcmp(cursor->attribute, attrbuf) == 0)
 			break;
+		}
 		if (cursor->attribute == NULL) {
 		    json_debug_trace((1,
 				      "Unknown attribute name '%s' (attributes begin with '%s').\n",
