@@ -1,9 +1,8 @@
-/* $Id: libgpsmm.cpp 6933 2010-01-13 22:46:47Z esr $ */
 /*
  * Copyright (C) 2005 Alfredo Pironti
  *
  * This software is distributed under a BSD-style license. See the
- * file "COPYING" for more information.
+ * file "COPYING" in the top-level directory of the disribution for details.
  *
  */
 #include <cstdlib>
@@ -56,6 +55,10 @@ struct gps_data_t* gpsmm::poll(void) {
 	}
 }
 
+int gpsmm::close(void) {
+	return gps_close(gps_data);
+}
+
 bool gpsmm::waiting(void) {
 	return gps_waiting(gps_data);
 }
