@@ -1,4 +1,7 @@
-/* $Id: strl.c 6185 2009-09-10 18:50:49Z esr $ */
+/*
+ * This file is Copyright (c) 2010 by the GPSD project
+ * BSD terms apply: see the file COPYING in the distribution root for details.
+ */
 #include <stdio.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
@@ -34,33 +37,33 @@
  * If retval >= siz, truncation occurred.
  */
 /*@ -usedef -mustdefine @*/
-size_t
-strlcat(char *dst, const char *src, size_t siz)
+size_t strlcat(char *dst, const char *src, size_t siz)
 {
-	char *d = dst;
-	const char *s = src;
-	size_t n = siz;
-	size_t dlen;
+    char *d = dst;
+    const char *s = src;
+    size_t n = siz;
+    size_t dlen;
 
-	/* Find the end of dst and adjust bytes left but don't go past end */
-	while (n-- != 0 && *d != '\0')
-		d++;
-	dlen = (size_t)(d - dst);
-	n = siz - dlen;
+    /* Find the end of dst and adjust bytes left but don't go past end */
+    while (n-- != 0 && *d != '\0')
+	d++;
+    dlen = (size_t) (d - dst);
+    n = siz - dlen;
 
-	if (n == 0)
-		return(dlen + strlen(s));
-	while (*s != '\0') {
-		if (n != 1) {
-			*d++ = *s;
-			n--;
-		}
-		s++;
+    if (n == 0)
+	return (dlen + strlen(s));
+    while (*s != '\0') {
+	if (n != 1) {
+	    *d++ = *s;
+	    n--;
 	}
-	*d = '\0';
+	s++;
+    }
+    *d = '\0';
 
-	return(dlen + (s - src));	/* count does not include NUL */
+    return (dlen + (s - src));	/* count does not include NUL */
 }
+
 /*@ +usedef +mustdefine @*/
 #endif /* HAVE_STRLCAT */
 
@@ -88,29 +91,28 @@ strlcat(char *dst, const char *src, size_t siz)
  * will be copied.  Always NUL terminates (unless siz == 0).
  * Returns strlen(src); if retval >= siz, truncation occurred.
  */
-size_t
-strlcpy(char *dst, const char *src, size_t siz)
+size_t strlcpy(char *dst, const char *src, size_t siz)
 {
-	char *d = dst;
-	const char *s = src;
-	size_t n = siz;
+    char *d = dst;
+    const char *s = src;
+    size_t n = siz;
 
-	/* Copy as many bytes as will fit */
-	if (n != 0) {
-		while (--n != 0) {
-			if ((*d++ = *s++) == '\0')
-				break;
-		}
+    /* Copy as many bytes as will fit */
+    if (n != 0) {
+	while (--n != 0) {
+	    if ((*d++ = *s++) == '\0')
+		break;
 	}
+    }
 
-	/* Not enough room in dst, add NUL and traverse rest of src */
-	if (n == 0) {
-		if (siz != 0)
-			*d = '\0';		/* NUL-terminate dst */
-		while (*s++ != '\0')
-			continue;
-	}
+    /* Not enough room in dst, add NUL and traverse rest of src */
+    if (n == 0) {
+	if (siz != 0)
+	    *d = '\0';		/* NUL-terminate dst */
+	while (*s++ != '\0')
+	    continue;
+    }
 
-	return((size_t)(s - src - 1));	/* count does not include NUL */
+    return ((size_t) (s - src - 1));	/* count does not include NUL */
 }
 #endif /* HAVE_STRLCPY */
