@@ -1,9 +1,8 @@
-/* $Id: test_gpsmm.cpp 6629 2009-11-30 17:23:36Z gdt $ */
 /*
  * Copyright (C) 2005 Alfredo Pironti
  *
  * This software is distributed under a BSD-style license. See the
- * file "COPYING" for more information.
+ * file "COPYING" in the top-level directory of the distribution for details.
  *
  */
 
@@ -16,7 +15,7 @@
 
 using namespace std;
 
-#if 0
+#ifdef SAMPLE_USAGE
 static void callback(struct gps_data_t* p, char* buf, size_t len);
 #endif
 
@@ -30,7 +29,7 @@ int main(void) {
 		return (1);
 	}
 
-#if 0
+#ifdef SAMPLE_USAGE
 	cout << "Going to set the callback...\n";
 	if (gps_rec.set_callback(callback)!=0 ) {
 		cout << "Error setting callback.\n";
@@ -41,8 +40,8 @@ int main(void) {
 	sleep(10);
 	cout << "Exited from sleep...\n";
 #endif
-	
-#if 0
+
+#ifdef SAMPLE_USAGE
 	if (gps_rec.del_callback()!=0) {
 		cout << "Error deleting callback\n";
 		return (1);
@@ -55,9 +54,8 @@ int main(void) {
 	return 0;
 }
 
-#if 0
+#ifdef SAMPLE_USAGE
 static void callback(struct gps_data_t* p, char* buf, size_t len) {
-	
 	if (p==NULL) {
 		cout << "Error polling gpsd\n";
 		return;
